import os, shutil, subprocess

'''A build system must have a "build" method with parameters "reconfigure" and "waitAfterConfig".'''

# Compile, build, and install cmake projects:
class CMake:
	def __init__(self, sourceFolder, buildFolder, config):
		self.sourceFolder = os.path.abspath(sourceFolder)
		self.buildFolder = os.path.abspath(buildFolder)
		self.config = config
	
	def build(self, reconfigure, waitAfterConfig):
		# Make sure we have a build directory
		if not os.path.exists(self.buildFolder): os.makedirs(self.buildFolder)
		os.chdir(self.buildFolder)
		# In case of reconfiguration, delete cache file if it exists
		cacheFile = 'CMakeCache.txt'
		if os.path.exists(cacheFile) and reconfigure: os.remove(cacheFile)
		# Run cmake
		os.putenv('PKG_CONFIG_PATH', os.path.join(self.config['installDir'], 'lib', 'pkgconfig')) # I found no way to do this within cmake
		os.putenv('CMAKE_PREFIX_PATH', self.config['installDir'])
		subprocess.check_call(['cmake', self.sourceFolder, '-DCMAKE_BUILD_TYPE='+self.config['buildType'],
			'-DCMAKE_INSTALL_PREFIX='+self.config['installDir']]+self.config.get('cmakeParameters', []))
		os.unsetenv('PKG_CONFIG_PATH')
		os.unsetenv('CMAKE_PREFIX_PATH')
		# if asked to do so, wait
		if waitAfterConfig:
			raw_input('Configuration done. Hit "Enter" to build the project. ')
		# run compilation
		subprocess.check_call(self.config.get('buildCmdPrefix', []) + ['make', '-j'+str(self.config['jobs'])])
		# run installation
		subprocess.check_call(self.config.get('installCmdPrefix', []) + ['make', 'install'])

# if auto-debuild is available, provide a wrapper for it
try:
	import auto_debuild
	class AutoDebuild:
		def __init__(self, sourceFolder, buildFolder, config, vcs):
			self.sourceFolder = os.path.abspath(sourceFolder)
			self.buildFolder = os.path.abspath(buildFolder)
			self.debFolder = os.path.abspath(config['debDir'])
			self.config = config
			self.vcs = vcs

		def build(self, reconfigure, waitAfterConfig): # reconfigure is ignored (we always do a reconfiguration)
			# create auto-debuild configuration
			versionName = self.config['versionName'] if 'versionName' in self.config else self.vcs.version()
			autoDebuildConfig = {
				'sourceName': self.config['name'],
				'buildSystem': self.config['buildSystem'],
				'debDir': self.debFolder,
				'buildDir': self.buildFolder,
				'name': self.config['debName'],
				'email': self.config['debEMail'],
				'parallelJobs': self.config['jobs'],
				'version': versionName,
				'waitAfterConfig': waitAfterConfig,
			}
			if autoDebuildConfig['version'] is None:
				raise Exception("VCS did not provide us with a proper version number, please fix this")
			# copy some more optional configuration
			for option in ('dbgPackage', 'section', 'withPython2', 'binarySkipFiles', 'binaryInstallFiles',
					'buildDepends', 'binaryDepends', 'binaryRecommends', 'binaryProvides', 'binaryConflicts', 'binaryBreaks',
					'binaryReplaces', 'binaryBreaksReplaces',
					'alternatives', 'cmakeParameters', 'automakeParameters'):
				if option in self.config:
					autoDebuildConfig[option] = self.config[option]
			# create Debian files
			os.chdir(self.sourceFolder)
			if os.path.isdir('debian'): # clean previous build attempts
				shutil.rmtree('debian')
			files = auto_debuild.createDebianFiles(autoDebuildConfig)
			# build package(s)
			auto_debuild.buildDebianPackage(autoDebuildConfig)
			# install package(s)
			if self.config.get('debInstall', True):
				subprocess.check_call(['sudo', 'dpkg', '--install'] + files)

except ImportError:
	#print "auto_debuild not found, disabling auto-debuild system"
	pass
